//*****************************************************************************
//*****************************************************************************
//  FILENAME:   feature.h
//  Version: 1.0, 2006/10/24 
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Handling of FEATURE requests
//
//  NOTE: The Toradex Definition is based on 32-Bytes Feature reports. 
// 		  Therefore, the file usb_descr.asm needs to be modified after each
//        "Generate Application":
//		  1) USB_INTERFACE_0_FEATURE_RPT_DATA: BLK  32
//		  2) USB_HID_RPT_1_FEATURE_RPT_SIZE:   EQU  32
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
#include <m8c.h>

typedef enum {AfterSampling=0, AfterChange=1, FixedRate=2} eReportMode;
typedef enum {eRAM=0, eFlash=1, eController=2, eSensor=3, eAction=0x80} eTarget;


//----------------------------------------
// Format definition for a FEATURE REPORT
// (union used to define different mappings for IN and OUT reports)
//----------------------------------------
typedef union
{
	struct
	{
		BOOL	Get_nSet;	// 0=set 1=get
		eTarget	Target;		// 0=RAM 1=Flash 2=PSoc-HW 3=Sensor, 0x80="Action"
		BYTE	ParamSize;	// 1=8bit, 2=16Bit ... 21=string
		BYTE	Index_LSB;	// Index / Address / Action Code
		BYTE	Index_MSB;	// "	(Little Endian)
		
		BYTE	Param[27];
	} OUT;

	struct
	{
		BYTE	ReturnCode;
		BYTE	Param[31];
	} IN;
} tFEATURE_RPT;

//----------------------------------------
// The actual FEATURE REPORT data
//----------------------------------------
extern tFEATURE_RPT	USB_INTERFACE_0_FEATURE_RPT_DATA;	
#define FEATURE_RPT	(USB_INTERFACE_0_FEATURE_RPT_DATA)

//-----------------------------------------------------------------------------
// Initialize Feature Reporting
//-----------------------------------------------------------------------------
extern void Feature_Start(void);

//-----------------------------------------------------------------------------
// check if feature report is pending
// if yes:
//	1) process request
//  2) prepare return data if appropriate
//  3) set FSM_FeatureRpt to allow further processing
//-----------------------------------------------------------------------------
void ProcessFeatureReports (void);

// end of file feature.h
