//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/11/12
//  Generated by Andy Kiser
//	Modified by Peter Lischer
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__

#include <m8c.h>        // part specific constants and macros
#include "string.h"
#include "sensor.h"
#include "psocgpioint.h"
#include "psocapi.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"


#ifdef __DEBUG__
#pragma ioport  UART_TX_CONTROL_REG:    0x02b               // (DxBxxCR0) Control register
BYTE            UART_TX_CONTROL_REG;
#pragma ioport  UART_TX_OUTPUT_REG: 0x12a                   // (DxBxxOU) Output register
BYTE            UART_TX_OUTPUT_REG;
#endif

//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00,		// LED mode
	0xF0,		// Direction Port A
	0x00,		// Direction Port B
	0xFF,		// Direction Port C
	0x00,		// Output Mode Port A
	0x00,		// Output Mode Port B
	0x00		// Output Mode Port C
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	50,			// report rate [ms]
	50,			// sample rate [ms]
	50000		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// Standby Configuration
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Oak IO",		// user device name
	"Frame Number", // user name Channel 0
	"I/O Port A", 	// user name Channel 1		
	"I/O Port B",	// user name Channel 2
	"I/O Port C", 	// user name Channel 3
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tInReport	InReport;
tOutReport	OutReport;
tInReport	InReport;
BYTE	NewSample;   // code in this file must set NewSample to TRUE, when new data is ready to be transferred to the host
BYTE	NofOutBytes; // number of bytes that were read from the last OUT report

extern BYTE SOF_Flag;
//----------------------------------------
// constant definitions
//----------------------------------------


//#define RELAY_MASK (RELAY0_MASK | RELAY1_MASK | RELAY2_MASK | RELAY3_MASK)

//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eOUT,
	eIN
} tFSM_Measure;



//----------------------------------------
// this module's global variables
//----------------------------------------

tFSM_Measure	FSM, StateAfterError;
WORD			Countdown_1ms;

//BYTE			IO_StandbyAction[3];
BYTE			OutputState[3];

// possible actions for each relay
//#define RLY_OFF			0
//#define RLY_ON			1
//#define RLY_TOGGLE		2
//#define RLY_NOCHANGE	3

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
//BYTE			DM2_2, DM2_1, DM2_0;
//BYTE			DM4_2, DM4_1, DM4_0;
//BYTE			DM5_2, DM5_1, DM5_0;
//BYTE			PORT_RELAY;

//----------------------------------------
// function prototypes
//----------------------------------------
//void Set_RelayPins(BYTE Action[4]);
void Set_DriveMode(void);

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start (void)
{
	BYTE i;
	
	//M8C_EnableIntMask(INT_MSK0,INT_MSK0_VOLTAGE_MONITOR); 	//enables the LVD interrupt for detecting 3.3V mode

	Set_DriveMode();
	IO_A0_Data_ADDR = 0x00;
	IO_B0_Data_ADDR = 0x00;
	IO_C0_Data_ADDR = 0x00;
	for(i=0; i<3;i++)
	{
		OutputState[i] = 0;
	}
	FSM = eOUT;

	// turn on power for the IO and pull up resistors
	GPIO_LOW(OFF_nON)
}

//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	typedef struct
	{
		BYTE Dummy;
		BYTE StbC;
		BYTE StbB;
		BYTE StbA;
	} tStbyMode;
	tStbyMode StbyMode;
	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	//DM2_2 = PRT2DM2;
	//DM2_1 = PRT2DM1;
	//DM2_0 = PRT2DM0;

	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// except the outputs that are configured as "not change for stand by"
	// OFF_nON is on Port0, all other signals on Port2
	PRT0DM2 |= ~OFF_nON_MASK;
	PRT0DM1 |= ~OFF_nON_MASK;
	PRT0DM0 &=  OFF_nON_MASK;

	Get_Param_32(PAR32_STBMODE, VOLATILE, (DWORD*)&StbyMode);
	//&StbyMode = (tStbyMode*) &PAR32_STBMODE;
	
	PRT2DM2  |= ~StbyMode.StbA; // leave stand by configured pins unmodified 
	PRT2DM1  |= ~StbyMode.StbA;
	PRT2DM0  &=  StbyMode.StbA;

	PRT4DM2  |= ~StbyMode.StbB; // leave stand by configured pins unmodified 
	PRT4DM1  |= ~StbyMode.StbB;
	PRT4DM0  &=  StbyMode.StbB;

	PRT5DM2  |= ~StbyMode.StbC; // leave stand by configured pins unmodified 
	PRT5DM1  |= ~StbyMode.StbC;
	PRT5DM0  &=  StbyMode.StbC;

	// turn off power for IO
	GPIO_HIGH(OFF_nON)
}

//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume(void)
{
	// restore drivemodes for GPIOs to/from the sensor
	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	Set_DriveMode();

	IO_A0_Data_ADDR = OutputState[0];
	IO_B0_Data_ADDR = OutputState[1];
	IO_C0_Data_ADDR = OutputState[2];
	// turn on power for the IO and pull up resistors
	GPIO_LOW(OFF_nON)
}
//-----------------------------------------------------------------------------
// Built-in self test
//-----------------------------------------------------------------------------
void BIST(void)
{
	WORD d;
	
	PRT2DM2  = 0xff; // Set All Pin as Output 
	PRT2DM1  = 0x00;
	PRT2DM0  = 0xff;
	
	PRT4DM2  = 0xff; // Set All Pin as Output 
	PRT4DM1  = 0x00;
	PRT4DM0  = 0xff;

	PRT5DM2  = 0xff; // Set All Pin as Output 
	PRT5DM1  = 0x00;
	PRT5DM0  = 0xff;
	for(;;)
	{

		IO_A0_Data_ADDR = 0x55;
		IO_B0_Data_ADDR = 0x55;
		IO_C0_Data_ADDR = 0x55;
		for (d=0; d<50000; d++);	
		IO_A0_Data_ADDR = 0xaa;
		IO_B0_Data_ADDR = 0xaa;
		IO_C0_Data_ADDR = 0xaa;
		for (d=0; d<50000; d++);	
	}
	
	// no useful function implemented yet
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	typedef struct
	{
		BYTE OUT1CMD;
		BYTE OUT2CMD;
	} tIOCmd;
	int i;
	tIOCmd *IOCmd;	
	// Start Builtin Self Test
	if  ( (!FEATURE_RPT.OUT.Get_nSet) && (FEATURE_RPT.OUT.Target == eAction) &&
	      (FEATURE_RPT.OUT.ParamSize == 0) && (FEATURE_RPT.OUT.Index_LSB == 0x80) )
	{
		BIST();
		return 1;
	}
	if  ( (FEATURE_RPT.OUT.Target == eController) &&
	      (FEATURE_RPT.OUT.ParamSize == 6) && (FEATURE_RPT.OUT.Index_LSB == 0x00) )
	{
		if (!FEATURE_RPT.OUT.Get_nSet)
		{
			for (i=0;i<3;i++)
			{
				IOCmd = (tIOCmd*)&FEATURE_RPT.OUT.Param[i+i];
				OutputState[i]= ((IOCmd->OUT1CMD & ~IOCmd->OUT2CMD) | (IOCmd->OUT1CMD & OutputState[i]) | (~IOCmd->OUT1CMD & IOCmd->OUT2CMD & ~OutputState[i]));
			}
			IO_A0_Data_ADDR = OutputState[0];
			IO_B0_Data_ADDR = OutputState[1];
			IO_C0_Data_ADDR = OutputState[2];
		}
		FEATURE_RPT.IN.Param[0] = IO_A0_Data_ADDR;
		FEATURE_RPT.IN.Param[1] = IO_B0_Data_ADDR;
		FEATURE_RPT.IN.Param[2] = IO_C0_Data_ADDR;
		return 3;
	}
	return 1;
}


//-----------------------------------------------------------------------------
// Set the output pins
//-----------------------------------------------------------------------------
void Set_DriveMode()
{
	BYTE DIRA,DIRB,DIRC,OMA,OMB,OMC;
	Get_Param_8(PAR8_DIRA, VOLATILE, &DIRA);
	Get_Param_8(PAR8_DIRB, VOLATILE, &DIRB);
	Get_Param_8(PAR8_DIRC, VOLATILE, &DIRC);
	Get_Param_8(PAR8_OMA, VOLATILE, &OMA);
	Get_Param_8(PAR8_OMB, VOLATILE, &OMB);
	Get_Param_8(PAR8_OMC, VOLATILE, &OMC);

	PRT2DM2  = DIRA;
	PRT2DM1  = ~DIRA | OMA;
	PRT2DM0  = DIRA;
	PRT4DM2  = DIRB;
	PRT4DM1  = ~DIRB | OMB;
	PRT4DM0  = DIRB;
	PRT5DM2  = DIRC;
	PRT5DM1  = ~DIRC | OMC;
	PRT5DM0  = DIRC;
/*
	DIR		OM	=>	DM2		DM1		DM0
	0		X		0		1		0		Input
	1		0		1		0		1		Output CMOS
	1		1		1		1		1		Output Open Drain
	
	DM2 = DIR
	DM1 = ~DIR | OM
	DM0 = DIR	
*/
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	if  ( (!FEATURE_RPT.OUT.Get_nSet) && (FEATURE_RPT.OUT.ParamSize == 1) &&
	      (FEATURE_RPT.OUT.Index_LSB >= 2) && (FEATURE_RPT.OUT.Index_LSB <= 7) )
		Set_DriveMode();
		
	NumberOfReturnedBytes = 0;
}

//-----------------------------------------------------------------------------
// Interrupt service routine for the trip volatge (LVD) detection 
// If the voltage is below 4V, this function handels the 3.3V operation mode
//-----------------------------------------------------------------------------
#pragma interrupt_handler LVD_ISR 
void LVD_ISR (void) 
{ 
}


//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure(void) 
{
	BYTE i;
	BYTE Mask;
	typedef struct
	{
		BYTE OUT1CMD;
		BYTE OUT2CMD;
	} tIOCmd;
	tIOCmd *IOCmd;	
	if (Countdown_1ms != 0)							// decrement millisecond counter
		Countdown_1ms--;
													// check if new instruction transmitted during currently active instruction
	// process actual state machine
	switch (FSM)
	{
		//----------------
		case eOUT: 
		//----------------
			if (NofOutBytes >0) 
			{
				NofOutBytes = 0;
				for (i=0;i<3;i++)
				{
					IOCmd = (tIOCmd*)&OutReport.IOData[i];
					OutputState[i]= ((IOCmd->OUT1CMD & ~IOCmd->OUT2CMD) | (IOCmd->OUT1CMD & OutputState[i]) | (~IOCmd->OUT1CMD & IOCmd->OUT2CMD & ~OutputState[i]));
					/*
						OUT2CMD		OUT1CMD		Old	=>	New
						0			0			0		0
						0			0			1		0
						0			1			0		1
						0			1			1		1
						1			0			0		1
						1			0			1		0
						1			1			0		0
						1			1			1		1
						
						New = (OUT1CMD & ~OUT2CMD) + (OUT1CMD & Old) + (~OUT1CMD & OUT2CMD & Old)
					*/
				}
				IO_A0_Data_ADDR = OutputState[0];
				IO_B0_Data_ADDR = OutputState[1];
				IO_C0_Data_ADDR = OutputState[2];
			}					
			if (Countdown_1ms == 0)
			{
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
				InReport.IOData[0] = IO_A0_Data_ADDR;
				InReport.IOData[1] = IO_B0_Data_ADDR;
				InReport.IOData[2] = IO_C0_Data_ADDR;
				NewSample = TRUE;
			}
			break;
	}
}


//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}

// end of sensor.c